/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Düsterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// +-
// |  some functions here rely upon this dialog to be openened by mnenhyFolderPanePopup!
// |  (in particular those using |opener|)
// +-

const gksPrefAttr = "mnenhy:pref";
const gksTypeAttr = "mnenhy:type";
const gksValAttr  = "mnenhy:value";
const gkoFS       = goMnenhy.folderstore;
const gksClicked  = gkoFS.gsClickedURI;

var   gsDataURI   = null;  // URI of the data folder
var   goCurCap    = null;  // current <caption> 
var   goDatCap    = null;  // data <caption> 
var   goDeck      = null;  // <deck>
var   goTreeGroup = null;  // <groupbox> with <tree>
var   goTree      = null;  // <tree>
var   goInTree    = null;  // main <treechildren>
var   goAddBC     = null;  // add <broadcaster>
var   goRemoveBC  = null;  // remove <broadcaster>
var   goEditBC    = null;  // edit <broadcaster>


function InitDialog()
{ // set the (clicked and data) folder's URI as the groupbox caption
  goCurCap = document.getElementById("mnenhyConfigCurrentCaption");
  goDatCap = document.getElementById("mnenhyConfigDataCaption");
  UpdateDataURI();
  // init tree
  goTree   = document.getElementById("mnenhyConfigTree");
  goInTree = document.getElementById("mnenhyConfigTreeChildren");
  UpdateTree();
  // init Add button
  goAddBC = document.getElementById("mnenhyConfigAddBC");
  UpdateAddBroadcaster();
  // init Remove button
  goRemoveBC = document.getElementById("mnenhyConfigRemoveBC");
  UpdateRemoveBroadcaster();
  // init Edit button
  goEditBC = document.getElementById("mnenhyConfigEditBC");
  UpdateEditBroadcaster();
  // init deck
  goTreeGroup = document.getElementById("mnenhyConfigDataGroup");
  goDeck = document.getElementById("mnenhyConfigDeck");
  UpdateDeck();
}


function UpdataDialog(abNoSizeToContent)
{
  UpdateDataURI();
  UpdateTree();
  UpdateAddBroadcaster();
  UpdateRemoveBroadcaster();
  UpdateEditBroadcaster();
  UpdateDeck(abNoSizeToContent);
}


function UpdateDeck(abNoSizeToContent)
{ // which deck panel do we need to show?
  // call UpdateDataURI and UpdateAddBroadcaster before this!
  if (gksClicked == gsDataURI)
  { // C. folder with own data
    goDeck.selectedIndex = 2;
    goTreeGroup.collapsed = false;
  }
  else if (goAddBC.getAttribute("disabled") == "true")
  { // A. folder inherits from global
    goDeck.selectedIndex = 0;
    goTreeGroup.collapsed = true;
  }
  else
  { // B. folder inherits from parent
    goDeck.selectedIndex = 1;
    goTreeGroup.collapsed = false;
  }
  if (!abNoSizeToContent)
    sizeToContent();
}


function UpdateDataURI()
{ // update the target URI
  gsDataURI = gkoFS.GetDataFolder(opener, gksClicked, true); // URI of the data folder
  goCurCap.setAttribute("label", decodeURIComponent(gksClicked));
}


function UpdateAddBroadcaster()
{ // We add prefs to the current data folder only
  goAddBC.setAttribute("disabled", gsDataURI == gkoFS.GetGlobalURI(opener));
}


// only allow removal if there's at least one item selected
function UpdateRemoveBroadcaster()
{ // we need to work around a bug in the aviary toolkit 
  // (in TB 0.9, <tree>.treeBoxObject.selection does not exist, but is used in
  //  <tree>.currentIndex - hence breaking goTree.currentIndex...)
  goRemoveBC.setAttribute("disabled", goTree.contentView.selection.currentIndex < 0);
}


// only allow edit if there's one item selected
function UpdateEditBroadcaster()
{ // we need to work around a bug in the aviary toolkit 
  // (in TB 0.9, <tree>.treeBoxObject.selection does not exist, but is used in
  //  <tree>.currentIndex - hence breaking goTree.currentIndex...)
  const kbNoEdit =    goTree.contentView.selection.currentIndex < 0
                   || goTree.contentView.selection.count != 1;
  goEditBC.setAttribute("disabled", kbNoEdit);
}


function UpdateTree()
{ // empty the tree
  while (goInTree.hasChildNodes())
    goInTree.removeChild(goInTree.lastChild);
  // read data, maybe from parent or global
  gkoFS.dd("UpdateTree: "+gsDataURI);
  if (gsDataURI)
  { // read the elements into the tree
    gkoFS.rdf.GetAllArcsOut(gsDataURI, this.ReadRDFArc);
  }
  goDatCap.setAttribute("label", decodeURIComponent(gsDataURI));
}
function ReadRDFArc(aoResourceIgnored, aoItem, aoValue)
{
  if (    (aoItem  instanceof goMnenhy.rdf.gkIRDFResource)
       && (aoValue instanceof goMnenhy.rdf.gkIRDFLiteral)
       && (!aoItem.Value.search(gkoFS.gkxTestNS))
     )
  { // copy the data into the tree
    AppendPrefToTree(aoItem.Value.replace(gkoFS.gkxTestNS, ""), aoValue.Value);
  }
  return true;
}


function AppendPrefToTree(asItem, asValue)
{
  gkoFS.dd("AppendPrefToTree("+asItem+","+asValue+")");
  var sType = "", iType = goMnenhy.kiPrefInvalid;
  switch (asValue.substr(0, gkoFS.gkiTypePrefixLen))
  {
    case gkoFS.gkoTypePrefix[goMnenhy.kiPrefString]: 
      sType = goTree.getAttribute("typestring"); 
      iType = goMnenhy.kiPrefString;
      break;
    case gkoFS.gkoTypePrefix[goMnenhy.kiPrefInt]:
      sType = goTree.getAttribute("typeint"); 
      iType = goMnenhy.kiPrefInt;
      break;
    case gkoFS.gkoTypePrefix[goMnenhy.kiPrefBool]:
      sType = goTree.getAttribute("typebool"); 
      iType = goMnenhy.kiPrefBool;
      break;
  }
  if (sType)
  {
    const koCell1 = document.createElement("treecell");
    koCell1.setAttribute("label", asItem);
    const koCell2 = document.createElement("treecell");
    koCell2.setAttribute("label", sType);
    const koCell3 = document.createElement("treecell");
    const ksValue = asValue.substr(gkoFS.gkiTypePrefixLen);
    koCell3.setAttribute("label", ksValue);
    const koRow = document.createElement("treerow")
    koRow.appendChild(koCell1);
    koRow.appendChild(koCell2);
    koRow.appendChild(koCell3);
    const koItem = document.createElement("treeitem")
    koItem.setAttribute(gksPrefAttr, asItem);
    koItem.setAttribute(gksTypeAttr, iType);
    koItem.setAttribute(gksValAttr,  ksValue);
    koItem.appendChild(koRow);
    goInTree.appendChild(koItem);
  }
}


function InheritFromParent(abCopyFromParent)
{ // change the inheritance mode for the clicked folder
  // abCopyFromParent denotes the action to be done
  if (abCopyFromParent)
  { // cancel inheritance by copying parent data
    opener.CopyFromParent();
  }
  else if (goMnenhy.ConfirmService(gkoFS.gksConfirmInherit, '', window))
  { // allow inheritance by deleting the now useless data
    gkoFS.rdf.RemoveContainer(gksClicked);
  }
  UpdataDialog();
}


function AddPref()
{ // Prefs will be added to the current data folder
  // Call UpdateAddBroadcaster() before this!
  if (gsDataURI)
  { // get new pref name
    var oParams = {pref: "", type: -1};
    window.openDialog("chrome://mnenhy-folderstore/content/mnenhy-folderstore-input.xul",
                      "mnenhyFolderstoreInput",
                      "chrome,modal,centerscreen",
                      oParams);
    if (oParams.pref)
    {
      const ksPrefRDF = goMnenhy.ksMnenhyNamespace + oParams.pref;
      if (gkoFS.rdf.GetItemValue(ksPrefRDF, gsDataURI) === null)
      { // add pref (but not twice)
        // read the current pref type and value
        var iType = goMnenhy.GetPrefType(oParams.pref);
        if (iType == goMnenhy.kiPrefInvalid)
          iType = oParams.type;
        const koValue = gkoFS.gkoTypePrefix[iType] + goMnenhy.GetPref(oParams.pref, iType);
        // add new pref to rdf
        gkoFS.rdf.ChangeItemValue(koValue, ksPrefRDF, gsDataURI);
        gkoFS.dd("AddPref: "+ksPrefRDF);
        UpdataDialog(true);
      } // add
    } // have new pref name
  } // have target
}


function RemovePref()
{ // will only be accessible when at least one pref is selected in the watch list
  var oStart = {}, oEnd = {};
  const kiCount = goTree.view.selection.getRangeCount();
  for (var i = 0; i < kiCount; ++i)
  { // loop over all ranges
    goTree.view.selection.getRangeAt(i, oStart, oEnd); // get selection range
    for (var j = oStart.value; j <= oEnd.value; ++j)
    { // delete all marked items
      const koTreeitem = goTree.view.getItemAtIndex(j);  // get the selected <treeitem>
      const ksPrefRDF  = goMnenhy.ksMnenhyNamespace + koTreeitem.getAttribute(gksPrefAttr);
      gkoFS.dd("RemovePref: "+ksPrefRDF);
      gkoFS.rdf.ChangeItemValue(null, ksPrefRDF, gsDataURI);  // delete it!
    } // items
  } // ranges
  UpdataDialog();
}


function EditPref()
{ // will only be accessible when one pref is selected in the watch list
  const kiCount = goTree.view.selection.getRangeCount();
  if (kiCount > 0)
  { // use first element of first selected range
    var oStart = {}, oEnd = {};
    goTree.view.selection.getRangeAt(0, oStart, oEnd); // get selection range
    const koTreeitem = goTree.view.getItemAtIndex(oStart.value);  // get the selected <treeitem>
    const ksPref  = koTreeitem.getAttribute(gksPrefAttr);
    var   iType   = koTreeitem.getAttribute(gksTypeAttr);
    var   sValue  = koTreeitem.getAttribute(gksValAttr);
    var   oParams = {pref: ksPref, type: iType, value: sValue, edit: true};
    window.openDialog("chrome://mnenhy-folderstore/content/mnenhy-folderstore-input.xul",
                      "mnenhyFolderstoreInput",
                      "chrome,modal,centerscreen",
                      oParams);
    iType = goMnenhy.GetPrefType(ksPref);
    if (iType == goMnenhy.kiPrefInvalid)
      iType = Number(oParams.type);
    // have valid type
    sValue = oParams.value;
    var oValue = "";
    switch (iType)
    {
      case goMnenhy.kiPrefBool:   
        oValue = sValue.replace(/^\s+|\s+$/g, "").toLowerCase() == "true";
        break;
      case goMnenhy.kiPrefInt:
        oValue = Number(sValue);
        if (isNaN(oValue))
          oValue = 0;
        break;
      case goMnenhy.kiPrefString:
      default:
        oValue = sValue;
        break;
    }
    const ksPrefRDF = goMnenhy.ksMnenhyNamespace + ksPref;
    sValue = gkoFS.gkoTypePrefix[iType] + oValue;
    // set changed pref
    gkoFS.rdf.ChangeItemValue(sValue, ksPrefRDF, gsDataURI);
    gkoFS.dd("EditPref: "+ksPrefRDF+"="+sValue);
    UpdataDialog(true);
  } // have selection
}


function ImportPrefs()
{
  gkoFS.ImportPrefs(gsDataURI);
  UpdataDialog();
}


function ExportPrefs()
{
  gkoFS.ExportPrefs(gsDataURI);
  UpdataDialog();
}

