/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002-2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Karsten Düsterloh <mnenhy@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// |
// |  constants
// |

// magic number
const gksMagicNumber               = "ADdv";
// header offsets
const gkiHDR_MAGIC                 =  0;
const gkiHDR_VERMAJOR              =  4;
const gkiHDR_VERMINOR              =  6;
const gkiHDR_AVAIL                 =  8;
const gkiHDR_ROOT                  = 12;
// key offsets
const gkiDESC_LOCATION             =  0;
const gkiDESC_NAME                 =  4;
const gkiDESC_NAMELEN              =  8;
const gkiDESC_TYPE                 = 10;
const gkiDESC_LEFT                 = 12;
const gkiDESC_DOWN                 = 16;
const gkiDESC_VALUE                = 20;
const gkiDESC_VALUELEN             = 24;
const gkiDESC_VALUEBUF             = 16;  // >= VALUELEN
const gkiDESC_PARENT               = 28;
/* types */
const gkiREGTYPE_KEY               = 0x0001;
const gkiREGTYPE_DELETED           = 0x0080;
const gkiREGTYPE_ENTRY             = 0x0010;
const gkiREGTYPE_ENTRY_STRING_UTF  = 0x0011;
const gkiREGTYPE_ENTRY_INT32_ARRAY = 0x0012;
const gkiREGTYPE_ENTRY_BYTES       = 0x0013;
const gkiREGTYPE_ENTRY_FILE        = 0x0014;

const gksMnenhyRegRigTree          = "mnenhyRegRigTreeChildren";
const gksPaddingZeros32            = "00000000";
const gksHexPrefix                 = "0x";
const gksSepSpace                  = " ";
const gksSepArrow                  = " --> ";

const gksXULTreeChildren           = "treechildren";
const gksXULTreeItem               = "treeitem";
const gksXULTreeRow                = "treerow";
const gksXULTreeCell               = "treecell";
const gksXULID                     = "id";
const gksXULLabel                  = "label";
const gksXULContainer              = "container";
const gksXULOpen                   = "open";

const gkoProperties                = goMnenhy.InitStringBundle("chrome://mnenhy/locale/mnenhy-regrig.properties")
const gksFilePickerTitle           = goMnenhy.GetString("FilePickerTitle",        gkoProperties);
const gksFilePickerMozProfTitle    = goMnenhy.GetString("FilePickerMozProfTitle", gkoProperties);
const gksFilePickerMozProfFile     = goMnenhy.GetString("FilePickerMozProfFile",  gkoProperties);
const gksFilePickerMozVerTitle     = goMnenhy.GetString("FilePickerMozVerTitle",  gkoProperties);
const gksFilePickerMozVerFile      = goMnenhy.GetString("FilePickerMozVerFile",   gkoProperties);
const gksFilePickerMozRegTitle     = goMnenhy.GetString("FilePickerMozRegTitle",  gkoProperties);
const gksFilePickerMozRegFile      = goMnenhy.GetString("FilePickerMozRegFile",   gkoProperties);
const gksFilePickerNSRegTitle      = goMnenhy.GetString("FilePickerNSRegTitle",   gkoProperties);
const gksFilePickerNSRegFile       = goMnenhy.GetString("FilePickerNSRegFile",    gkoProperties);
const gksFilePickerAllTitle        = goMnenhy.GetString("FilePickerAllTitle",     gkoProperties);
const gksFilePickerAllFile         = goMnenhy.GetString("FilePickerAllFile",      gkoProperties);
const gksErrorNoMozRegFile         = goMnenhy.GetString("ErrorNoMozRegFile",      gkoProperties);

// |
// |  vars
// |
var goRoot = null;


// windows defaults:
// "%appdata%/Mozilla/registry.dat"
// "%windir%/mozver.dat"
// "%windir%/mozregistry.dat"


// |
// |  readability functions
// |

// convert string of little endian bytes into integer
function str2int(s)
{
  var iRet = -1;
  if (s)
  {
    iRet = 0;
    for (var i = s.length - 1; i >= 0; --i)
    {
      iRet *= 0x100;
      iRet += s.charCodeAt(i);
    }
  }
  return iRet;
}

// convert character c into hex string of length len (default = 2)
function char2hex(c, len)
{
  if (!len)
    len = -2;
  else if (len > 0)
    len = -len;
  if (c)
    return (gksPaddingZeros32 + c.charCodeAt(0).toString(16)).substr(len);
  else
    return "";
}

// convert string of little endian bytes into space delimited hex string
function str2hex(s)
{
  var sRet = "";
  if (s)
  {
    sRet = char2hex(s[0]);
    for (var i = 1; i < s.length; ++i)
      sRet += gksSepSpace + char2hex(s[i]);
  }
  return sRet;
}

// convert string of little endian 32bit integer into hex string
function str2inthex1(s)
{
  var sRet = "";
  if (s)
  {
    for (var i = 3; i >= 0; --i)
      sRet += char2hex(s[i], 2);
  }
  return sRet;
}

// convert string of little endian 32bit integers into space delimited hex string
function str2inthex(s)
{
  var sRet = "";
  if (s)
  {
    sRet = str2inthex1(s.substr(0,4));
    for (var i = 4; i < s.length; i += 4)
      sRet += gksSepSpace + str2inthex1(s.substr(i,4));
  }
  return sRet;
}

// convert integer into hex string with leading 0x
function int2str(i)
{
  return gksHexPrefix + i.toString(16);
}


// create tree item with values and append it to aoParent
function CreateTreeItem(aoParent, aiNode, asKey, aiType, asValue, aoChildren)
{
  var oItem         = document.createElement(gksXULTreeItem);
  var oItemRow      = document.createElement(gksXULTreeRow);
  var oItemCellNode = document.createElement(gksXULTreeCell);
  var oItemCellKey  = document.createElement(gksXULTreeCell);
  var oItemCellType = document.createElement(gksXULTreeCell);
  var oItemCellVal  = document.createElement(gksXULTreeCell);
  if (aiNode >= 0)
  {
    var sNode = int2str(aiNode);
    oItem        .setAttribute(gksXULID,    sNode);
    oItem        .setAttribute(gksXULOpen,  (aoParent == goRoot));
    oItemCellNode.setAttribute(gksXULLabel, sNode);
  }
  oItemCellKey .setAttribute(gksXULLabel, asKey);
  if (aiType >= 0)
  {
    oItemCellType.setAttribute(gksXULLabel, int2str(aiType));
  }
  oItemCellVal .setAttribute(gksXULLabel, asValue);
  oItemRow.appendChild(oItemCellNode);
  oItemRow.appendChild(oItemCellKey);
  oItemRow.appendChild(oItemCellType);
  oItemRow.appendChild(oItemCellVal);
  oItem   .appendChild(oItemRow);
  if (aoChildren && aoChildren.childNodes.length)
  {
    oItem.setAttribute(gksXULContainer, true)
    oItem.appendChild(aoChildren);
  }
  aoParent.appendChild(oItem);
}


// create one <tree> node and attach it to aoParent
function DumpNode(aoData, aiNode, aoParent)
{
  var iValue;
  while (aiNode)
  {
    var iType = str2int(aoData.sData.substr(aiNode + gkiDESC_TYPE, 2));
    if (iType != gkiREGTYPE_DELETED)
    {
      var iName     = str2int(aoData.sData.substr(aiNode + gkiDESC_NAME,    4));
      var iNameLen  = str2int(aoData.sData.substr(aiNode + gkiDESC_NAMELEN, 2));
      var sName     = aoData.sData.substr(iName, iNameLen - 1);
      var sValue    = "";
      var oChildren = null;

      if (iType == gkiREGTYPE_KEY)
      {
        oChildren = document.createElement(gksXULTreeChildren);
        // value?
        iValue = str2int(aoData.sData.substr(aiNode + gkiDESC_VALUE, 4));
        if (iValue)
          DumpNode(aoData, iValue, oChildren);

        // go down!
        var iDown = str2int(aoData.sData.substr(aiNode + gkiDESC_DOWN, 4));
        if (iDown)
          DumpNode(aoData, iDown, oChildren);
      }
      else
      { // value entry!
        iValue = str2int(aoData.sData.substr(aiNode + gkiDESC_VALUE,    4));
        var iValueLen = str2int(aoData.sData.substr(aiNode + gkiDESC_VALUELEN, 4));

        switch (iType)
        {
          case gkiREGTYPE_ENTRY_STRING_UTF:
          case gkiREGTYPE_ENTRY_FILE:
            sValue = aoData.sData.substr(iValue, iValueLen-1);
            break;

          case gkiREGTYPE_ENTRY_INT32_ARRAY:
            sValue = str2inthex(aoData.sData.substr(iValue, iValueLen));
            break;

          case gkiREGTYPE_ENTRY_BYTES:
            {
              var sh = aoData.sData.substr(iValue, iValueLen);
              sValue = str2hex(sh) + gksSepArrow + sh;
            }
            break;
        } // switch
      } // value

      // go create!
      CreateTreeItem(aoParent, aiNode, sName, iType, sValue, oChildren);

      // go left!
      aiNode = str2int(aoData.sData.substr(aiNode + gkiDESC_LEFT, 4));
    } // not deleted
  } // while
}


// Create the tree view of a Mozilla registry file
function InitTree()
{ // get the registry file to open
  const koFilePicker = Components.interfaces.nsIFilePicker;
  var oFP = Components.classes[goMnenhy.ksCID_FilePicker].createInstance(koFilePicker);
  oFP.init(window, gksFilePickerTitle, koFilePicker.modeOpen);
  oFP.appendFilter(gksFilePickerMozProfTitle, gksFilePickerMozProfFile);
  oFP.appendFilter(gksFilePickerMozVerTitle,  gksFilePickerMozVerFile);
  oFP.appendFilter(gksFilePickerMozRegTitle,  gksFilePickerMozRegFile);
  oFP.appendFilter(gksFilePickerNSRegTitle,   gksFilePickerNSRegFile);
  oFP.appendFilter(gksFilePickerAllTitle,     gksFilePickerAllFile);
  oFP.displayDirectory = goMnenhy.GetSystemFile("AppRegD");
  oFP.defaultString    = gksFilePickerMozProfFile;
  if (oFP.show() == koFilePicker.returnOK)
  {
    var oMozVerDatFile = oFP.file;
    if (oMozVerDatFile.exists())
    {
      goMnenhy.SetWaitCursor(window);
      var isMozVerDat = goMnenhy.GetBinStream(oMozVerDatFile);
      var iSize       = isMozVerDat.available();
      var oMozVerDat  = {sData: isMozVerDat.readBytes(iSize)};  // avoid copying the string
      goRoot = document.getElementById(gksMnenhyRegRigTree);
      // check magic key
      if (oMozVerDat.sData.substr(gkiHDR_MAGIC, 4) == gksMagicNumber)
      { // file is MozReg file
        var iRoot = str2int(oMozVerDat.sData.substr(gkiHDR_ROOT, 4));
        DumpNode(oMozVerDat, iRoot, goRoot);
      }
      else
      {
        CreateTreeItem(goRoot, -1, gksErrorNoMozRegFile, -1, "", null);
      }
      goMnenhy.UnsetWaitCursor(window);
    } // exists
  } // file picked
  else
    close();
}
